<?php

namespace Magento\FDPPaymentGateway\Block;

class RedoPay extends \Magento\Framework\View\Element\Template
{	
	protected $scopeConfig;
	protected $cdhelper;
	protected $visible = false;
	
	public function __construct(
		\Magento\Framework\View\Element\Template\Context $context,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\FDPPaymentGateway\Helper\Cdata $cdhelper
	)
	{
		$this->scopeConfig = $scopeConfig;
		$this->cdhelper = $cdhelper;
		parent::__construct($context);
	}

	public function getGatewayUrl() {
		return $this->scopeConfig->getValue('payment/fdp_gateway/gateway');
	}

	public function isVisible() {
		return $this->visible;
	}
	
	private function genData($id) {
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$store = $objectManager->get('Magento\Framework\Locale\Resolver');
		$order = $objectManager->create('Magento\Sales\Model\Order')->load($id);
		if($order->getId() == 0) return false;
		$adr = $order->getBillingAddress();
		$d = array(
			'pos_id' => $this->scopeConfig->getValue('payment/fdp_gateway/pos_id'),
			'order_id' => $id,
			'session_id' => md5($id . $this->scopeConfig->getValue('payment/fdp_gateway/key')),
			'amount' => strval(round(floatval($order->getBaseGrandTotal()) * 100)),
			'currency' => $order->getBaseCurrencyCode(),
			'test' => $this->scopeConfig->getValue('payment/fdp_gateway/test') == '1' ? 'Y' : 'N',
			'language' => substr($store->getLocale(), 3, 2),
			'client_ip' => $_SERVER['REMOTE_ADDR'],
			'street' => '',
			'street_n1' => '',
			'street_n2' => '',
			'city' => '',
			'postcode' => '',
			'country' => $adr->getCountryId(),
			'email' => substr($order->getCustomerEmail(), 0, 40),
			'ba_firstname' => '',
			'ba_lastname' => ''
		);

		$k = $this->scopeConfig->getValue('payment/fdp_gateway/key');
		$cdata = $this->cdhelper->getCdata($d, $k);
		$d['controlData'] = $cdata;
		$this->visible = true;
		return $d;
	}

	private function _calcControlData($d, $sharedKey)
	{
		$data = array();
		foreach($d as $k => $w) if($w) $data[$k] = $w;
		# build query
		$query = '';
		foreach($data as $key => $value)
			$query .= ($query ? '&' : '') . $key .'='. $value;
	
		# add shared key binary string
		$l = strlen($sharedKey)/2;
		$sharedKeyBin = '';
		for($i=1; $i<=$l; ++$i)
			$sharedKeyBin .= pack('H*', substr($sharedKey, 2*$i-2, 2));
	
		$query .= $sharedKeyBin;
	
		# hash
		return hash('sha256', $query);
	}
	
	private function verify() {
		if(!isset($_POST['order_id'])) return true;
		if(isset($_POST['err_code'])) return false;
		$data = array(
			'pos_id'			=> (int) $_POST['pos_id'],
			'order_id'			=> (int) $_POST['order_id'],
			'session_id'		=> $_POST['session_id'],
			'amount'			=> (int) $_POST['amount'],
			'response_code'		=> (int) $_POST['response_code'],
			'transaction_id'	=> $_POST['transaction_id'],
			'cc_number_hash'	=> isset($_POST['cc_number_hash']) ? $_POST['cc_number_hash'] : null,
			'bin'				=> isset($_POST['bin']) ? $_POST['bin'] : null,
			'card_type'			=> isset($_POST['card_type']) ? $_POST['card_type'] : null,
			'auth_code'			=> isset($_POST['auth_code']) ? $_POST['auth_code'] : null,
		);
		
		# kod kontrolny
		$controlData = $_POST['controlData'];
		$calcControlData = $this->_calcControlData($data, $this->scopeConfig->getValue('payment/fdp_gateway/key'));
		
		return (($controlData == $calcControlData));
	}

	public function genForm()
	{
		if(!isset($_POST['order_id']) && !isset($_GET['id'])) return '<input type="hidden" name="redo_error" value="1">';
		if(!$this->verify()) return '<input type="hidden" name="redo_error" value="2">';
		$id = isset($_POST['order_id']) ? (int) $_POST['order_id'] : (int) $_GET['id'];
		$arr = $this->genData($id);
		if(!$arr) return '<input type="hidden" name="redo_error" value="3">';
		$form = '';
		foreach ($arr as $k => $v) {
			$v = str_replace('"', '\"', $v);
			$form .= '<input type="hidden" name="' . $k . '" value="' . $v . '">';
		};
		return $form;
	}

	
}

?>